###################################################################################################
# _ftpmanager.py
#
# $Id: _ftpmanager.py,v 1.1 2003/08/31 13:29:22 dnordmann Exp $
# $Name:  $
# $Author: dnordmann $
# $Revision: 1.1 $
#
# Implementation of class _ftpmanager.FtpManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTMLFile
import urllib
import ftplib
import time
import os
import stat
# Product Imports.
import _fileutil
import _globals


# ---------------------------------------------------------------------------------------------
#  _ftpmanager.recurse_Local:
# ---------------------------------------------------------------------------------------------
def recurse_Local(ftp, remoteFolder='', localFolder='', sPath=''):
  #++ print '[recurse_Local]: Path="%s"'%(sPath)
  for sFile in os.listdir(localFolder+sPath+'/'):
    fStat = os.stat(localFolder+sPath+'/'+sFile)
    fSize = fStat[stat.ST_SIZE]
    fMode = fStat[stat.ST_MODE]
    if stat.S_ISDIR(fMode):
      # Change to new directory (if it does not exist it will be created).
      ftpCwd(ftp,remoteFolder+sPath,sFile)
      # Recurse.
      recurse_Local(ftp,remoteFolder,localFolder,sPath+'/'+sFile)
      # Change back to current directory.
      ftp.cwd(remoteFolder+sPath+'/')
    else:
      # Delete existing file.
      try:
        ftp.delete(sFile)
      except: pass
      # Create new file.
      try:
        file = open(localFolder+sPath+'/'+sFile,'rb')
        ftp.storbinary('STOR %s'%sFile,file,fSize)
        file.close()
      except: pass


# ---------------------------------------------------------------------------------------------
#  _ftpmanager.ftpCwd:
#	
#  Change work directory on ftp-host.
# ---------------------------------------------------------------------------------------------
def ftpCwd(ftp, path, dir):
  #++ print '[ftpCwd]: %s/%s'%(path,dir)
  # Change to new directory.
  try:
    ftp.cwd(path + '/' + dir)
  # Create new directory.
  except:
    try:
      ftp.mkd(dir)
    except:
      pass
    ftp.cwd(path + '/' + dir)


# ---------------------------------------------------------------------------------------------
#  _ftpmanager.ftpToProvider:
# ---------------------------------------------------------------------------------------------
def ftpToProvider(self, lang, manage_lang, REQUEST, RESPONSE):
    message = ''
    
    # Profile time.
    tStart = time.time()
    
    # Create temporary local-folder.
    s_tmp_folder = self.__prepare_temp_export_folder__(from_content=1,from_zms=1,from_common=1,REQUEST=REQUEST)
    # Download HTML-pages to temporary local-folder.
    # << vy 20040831 Don't remove /content/-Element
    #self.recurse_downloadHtmlPages(self,s_tmp_folder,REQUEST)
    self.recurse_downloadHtmlPages(self,'%s/%s'%(s_tmp_folder,'content'),REQUEST,fp)
    # >> vy 20040831 Don't remove /content/-Element
    
    # Connect to FTP-server.
    # ----------------------
    dctFtp = self.getFtp(REQUEST)
    try:
      ftp = ftplib.FTP(dctFtp['site'])
      ftp.set_debuglevel(1) # 0=no, 1=moderate, 2=maximum debugging output
      ftp.login(dctFtp['userid'],dctFtp['password'])
      ftpCwd(ftp,'',dctFtp['path'])
      recurse_Local(ftp,dctFtp['path'],s_tmp_folder)
      message += ftp.getwelcome()+'<br>'
      ftp.quit()
      
    except:
      message += _globals.writeException(self,"[_ftpmanager.ftpToProvider]:")+'<br>'
    
    # Remove temporary local-folder.
    _fileutil.remove(s_tmp_folder,deep=1)
    
    # Return with message.
    message += self.getLangStr('MSG_EXPORTED',manage_lang)%('%s <b>%s</b> in %d sec.'%(self.display_type(REQUEST),dctFtp['site']+dctFtp['path'],(time.time()-tStart)))
    return message


# ---------------------------------------------------------------------------------------------
#  _ftpmanager.exportToLocal:
# ---------------------------------------------------------------------------------------------
# << vy 20040609 trace local export to file in client_home, try to catch all exceptions
def exportToLocal(self, lang, manage_lang, REQUEST, RESPONSE):
    message = ''
    
    # Profile time.
    tStart = time.time()
    
    try:
      client_home = self.aq_acquire('Control_Panel').getCLIENT_HOME()
      zms_instance_name = self.getDocumentElement().getHome().getId()
      filename = os.path.join(client_home, 'zms.exportToLocal.' + zms_instance_name + '.2.log')
      fd = os.open(filename, os.O_WRONLY|os.O_CREAT|os.O_EXCL, 0664)
      fp = os.fdopen(fd, 'w')
      fp.write('exportToLocal started:' + time.strftime('%Y%m%d-%H%M%S', time.gmtime()) + ':/started\n')

      # Create temporary local-folder.
      # << vy 20041028 Trace path to fp while recursing
      #s_tmp_folder = self.__prepare_temp_export_folder__(from_content=1,from_zms=1,from_common=1,REQUEST=REQUEST)
      s_tmp_folder = self.__prepare_temp_export_folder__(from_content=1,from_zms=1,from_common=1,REQUEST=REQUEST,fp=fp)
      # >> vy 20041028 Trace path to fp while recursing
      fp.write('exportToLocal tmp_folder:' + s_tmp_folder + ':/tmp_folder\n')
      fp.flush()

      # Download HTML-pages to temporary local-folder.
      # << vy 20040831 Don't remove /content/-Element
      #self.recurse_downloadHtmlPages(self,s_tmp_folder,REQUEST,fp)
      self.recurse_downloadHtmlPages(self,'%s/%s'%(s_tmp_folder,'content'),REQUEST,fp)
      # >> vy 20040831 Don't remove /content/-Element

      fp.write('exportToLocal success:' + time.strftime('%Y%m%d-%H%M%S', time.gmtime()) + ':/success\n')
      fp.flush()
      fp.write('exportToLocal finished:' + time.strftime('%Y%m%d-%H%M%S', time.gmtime()) + ':/finished\n')
      fp.flush()
      fp.close()
    except: # file could not be opened.
      msg = _globals.writeException(self,"[_ftpmanager.ftpToProvider]:")+'<br>'
      message += msg
      try:
        fp.write('exportToLocal failed:' +time.strftime('%Y%m%d-%H%M%S', time.gmtime()) + ':/failed\n')
        fp.flush()
        fp.write('exportToLocal msg:' + msg + ':/msg\n')
        fp.flush()
        fp.write('exportToLocal finished:' + time.strftime('%Y%m%d-%H%M%S', time.gmtime()) + ':/finished\n')
        fp.flush()
        fp.close()
      except: # fp might not have been bound, so ignore the exception
        pass
      return message # anyway, return a message

    # DO NOT Remove temporary local-folder; we need it later in stage.sh
    # _fileutil.remove(s_tmp_folder,deep=1)
    
    # Return with message (displayed on management interface; will be ignored).
    message += self.getLangStr('MSG_EXPORTED',manage_lang)%('%s <b>%s</b> in %d sec.'%(self.display_type(REQUEST),'local',(time.time()-tStart)))
    return message
# >> vy 20040609 trace local export to file in client_home, try to catch all exceptions


###################################################################################################
###################################################################################################
###
###   class FtpManager
###
###################################################################################################
###################################################################################################
class FtpManager: 

    # Management Interface.
    # ---------------------
    manage_importexportFtp = HTMLFile('dtml/zmscontainerobject/manage_importexportftp', globals()) 


    # ---------------------------------------------------------------------------------------------
    #  FtpManager.getFtp: 
    #
    #  Get parameters of FTP access to provider.
    # ---------------------------------------------------------------------------------------------
    def getFtp(self, REQUEST): 
      if getattr(self,'attr_provider_ftp_site','') and \
      	 getattr(self,'attr_provider_ftp_userid','') and \
      	 getattr(self,'attr_provider_ftp_password',''):
        rtn = {}
        rtn['site'] = getattr(self,'attr_provider_ftp_site','')
        rtn['path'] = getattr(self,'attr_provider_ftp_path','')
        rtn['userid'] = getattr(self,'attr_provider_ftp_userid','')
        rtn['password'] = getattr(self,'attr_provider_ftp_password','')
        return rtn
      return None         

    ###############################################################################################
    # FtpManager.manage_customizeFtp: 
    #
    # Change parameters of FTP access to provider.
    ###############################################################################################
    def manage_customizeFtp(self, btn, lang, manage_lang, REQUEST, RESPONSE):
      """ FtpManager.manage_customizeFtp """
      
      message = ''
      
      # Change.
      # -------
      self.attr_provider_ftp_site = REQUEST['site']
      self.attr_provider_ftp_path = REQUEST['path']
      self.attr_provider_ftp_userid = REQUEST['userid']
      self.attr_provider_ftp_password = REQUEST['password']
      message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Ping.
      # -----
      if btn == self.getLangStr('BTN_PING',manage_lang):
        try:
          # Profile time.
          tStart = time.time()
          ftp = ftplib.FTP(self.attr_provider_ftp_site)
          ftp.set_debuglevel(1) # moderate output
          ftp.login(self.attr_provider_ftp_userid,self.attr_provider_ftp_password)
          message = '%s%s<br>'%(message,ftp.getwelcome())
          ftp.quit()
          message = 'Ping in %d sec.'%(time.time()-tStart)
        except:
          message = _globals.writeException(self,"[manage_customizeFtp]:")
      
      # Export.
      # -------
      if btn == self.getLangStr('BTN_EXPORT',manage_lang):
        message = ftpToProvider(self,lang,manage_lang,REQUEST,RESPONSE)

      # << vy 20040609 trace local export to file in client_home, try to catch all exceptions
      # Export to local folder only.
      # -------
      if btn == 'BTN_EXPORT_LOCAL':
        message = exportToLocal(self,lang,manage_lang,REQUEST,RESPONSE)
      # >> vy 20040609 trace local export to file in client_home, try to catch all exceptions

      # Return with message.
      return REQUEST.RESPONSE.redirect('manage_importexportFtp?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,urllib.quote(message)))

###################################################################################################
